@@ -28,16 +28,24 @@
 
 #include <string.h>
 
+#include <xf86.h>
 #include <xf86_OSproc.h>
+#if GET_ABI_MAJOR(ABI_VIDEODRV_VERSION) < 6
 #include <xf86Resources.h>
-#include <xf86RandR12.h>
+#endif
 #include <mipointer.h>
 #include <mibstore.h>
 #include <micmap.h>
 #include <xf86cmap.h>
 #include <fb.h>
+#ifdef HAVE_XEXTPROTO_71
+#include <X11/extensions/dpmsconst.h>
+#else
 #define DPMS_SERVER
 #include <X11/extensions/dpms.h>
+#endif
+
+#include <xf86RandR12.h>
 
 #include "nv_const.h"
 #include "g80_type.h"
@@ -51,56 +59,6 @@
 #define G80_REG_SIZE (1024 * 1024 * 16)
 #define G80_RESERVED_VIDMEM 0xd000
 
-static const char *fbSymbols[] = {
-    "fbPictureInit",
-    "fbScreenInit",
-    NULL
-};
-
-static const char *xaaSymbols[] = {
-    "XAACopyROP",
-    "XAACreateInfoRec",
-    "XAADestroyInfoRec",
-    "XAAFallbackOps",
-    "XAAInit",
-    "XAAPatternROP",
-    NULL
-};
-
-static const char *exaSymbols[] = {
-    "exaDriverAlloc",
-    "exaDriverInit",
-    "exaDriverFini",
-    NULL
-};
-
-static const char *i2cSymbols[] = {
-    "xf86CreateI2CBusRec",
-    "xf86I2CBusInit",
-    NULL
-};
-
-static const char *ramdacSymbols[] = {
-    "xf86CreateCursorInfoRec",
-    "xf86DestroyCursorInfoRec",
-    "xf86InitCursor",
-    NULL
-};
-
-static const char *ddcSymbols[] = {
-    "xf86PrintEDID",
-    "xf86DoEDID_DDC2",
-    "xf86SetDDCproperties",
-    NULL
-};
-
-static const char *int10Symbols[] = {
-    "xf86FreeInt10",
-    "xf86InitInt10",
-    "xf86ExecX86int10",
-    NULL
-};
-
 typedef enum {
     OPTION_HW_CURSOR,
     OPTION_NOACCEL,
@@ -243,7 +201,6 @@ G80PreInit(ScrnInfoPtr pScrn, int flags)
     pNv->int10 = NULL;
     pNv->int10Mode = 0;
     if(xf86LoadSubModule(pScrn, "int10")) {
-        xf86LoaderReqSymLists(int10Symbols, NULL);
         xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Initializing int10\n");
         pNv->int10 = xf86InitInt10(pEnt->index);
     }
@@ -275,8 +232,10 @@ G80PreInit(ScrnInfoPtr pScrn, int flags)
     }
 
     /* Disable VGA access */
+#ifndef XSERVER_LIBPCIACCESS
     xf86SetOperatingState(resVgaIo, pEnt->index, ResUnusedOpr);
     xf86SetOperatingState(resVgaMem, pEnt->index, ResDisableOpr);
+#endif
 
     pScrn->monitor = pScrn->confScreen->monitor;
 
@@ -360,11 +319,13 @@ G80PreInit(ScrnInfoPtr pScrn, int flags)
     xf86DrvMsg(pScrn->scrnIndex, X_INFO, "MMIO registers mapped at %p\n",
                (void*)pNv->reg);
 
+#ifndef XSERVER_LIBPCIACCESS
     if(xf86RegisterResources(pEnt->index, NULL, ResExclusive)) {
         xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "xf86RegisterResources() found "
                    "resource conflicts\n");
         goto fail;
     }
+#endif
 
     pNv->architecture = pNv->reg[0] >> 20 & 0x1ff;
     tmp = pNv->reg[0x0010020C/4];
@@ -434,7 +395,6 @@ G80PreInit(ScrnInfoPtr pScrn, int flags)
 
     if(!xf86LoadSubModule(pScrn, "i2c")) goto fail;
     if(!xf86LoadSubModule(pScrn, "ddc")) goto fail;
-    xf86LoaderReqSymLists(i2cSymbols, ddcSymbols, NULL);
 
     if(!G80DispPreInit(pScrn)) goto fail;
     /* Read the DDC routing table and create outputs */
@@ -465,17 +425,14 @@ G80PreInit(ScrnInfoPtr pScrn, int flags)
 
     /* Load fb */
     if(!xf86LoadSubModule(pScrn, "fb")) goto fail;
-    xf86LoaderReqSymLists(fbSymbols, NULL);
 
     if(!pNv->NoAccel) {
         switch(pNv->AccelMethod) {
         case XAA:
             if(!xf86LoadSubModule(pScrn, "xaa")) goto fail;
-            xf86LoaderReqSymLists(xaaSymbols, NULL);
             break;
         case EXA:
             if(!xf86LoadSubModule(pScrn, "exa")) goto fail;
-            xf86LoaderReqSymLists(exaSymbols, NULL);
             break;
         }
     }
@@ -486,9 +443,7 @@ G80PreInit(ScrnInfoPtr pScrn, int flags)
             xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "Failed to load ramdac. "
                        "Falling back to software cursor.\n");
             pNv->HWCursor = FALSE;
-        } else {
-            xf86LoaderReqSymLists(ramdacSymbols, NULL);
-        }
+	}
     }
 
     return TRUE;
