@@ -124,7 +124,7 @@ _X_EXPORT DriverRec NV = {
 #endif
 };
 
-/* Known cards as of 2008/08/26 */
+/* Known cards as of 2009/05/15 */
 
 static SymTabRec NVKnownChipsets[] =
 {
@@ -288,6 +288,7 @@ static SymTabRec NVKnownChipsets[] =
   { 0x10DE0147, "GeForce 6700 XL" },
   { 0x10DE0148, "GeForce Go 6600" },
   { 0x10DE0149, "GeForce Go 6600 GT" },
+  { 0x10DE014A, "Quadro NVS 440" },
   { 0x10DE014C, "Quadro FX 550" },
   { 0x10DE014D, "Quadro FX 550" },
   { 0x10DE014E, "Quadro FX 540" },
@@ -322,6 +323,7 @@ static SymTabRec NVKnownChipsets[] =
   { 0x10DE0099, "GeForce Go 7800 GTX" },
   { 0x10DE009D, "Quadro FX 4500" },
 
+  { 0x10DE01D0, "GeForce 7350 LE" },
   { 0x10DE01D1, "GeForce 7300 LE" },
   { 0x10DE01D3, "GeForce 7300 SE" },
   { 0x10DE01D6, "GeForce Go 7200" },
@@ -335,6 +337,7 @@ static SymTabRec NVKnownChipsets[] =
   { 0x10DE01DE, "Quadro FX 350" },
   { 0x10DE01DF, "GeForce 7300 GS" },
 
+  { 0x10DE0390, "GeForce 7650 GS" },
   { 0x10DE0391, "GeForce 7600 GT" },
   { 0x10DE0392, "GeForce 7600 GS" },
   { 0x10DE0393, "GeForce 7300 GT" },
@@ -351,6 +354,10 @@ static SymTabRec NVKnownChipsets[] =
   { 0x10DE0290, "GeForce 7900 GTX" },
   { 0x10DE0291, "GeForce 7900 GT" },
   { 0x10DE0292, "GeForce 7900 GS" },
+  { 0x10DE0293, "GeForce 7950 GX2" },
+  { 0x10DE0294, "GeForce 7950 GX2" },
+  { 0x10DE0295, "GeForce 7950 GT"},
+  { 0x10DE0297, "GeForce Go 7950 GTX" },
   { 0x10DE0298, "GeForce Go 7900 GS" },
   { 0x10DE0299, "GeForce Go 7900 GTX" },
   { 0x10DE029A, "Quadro FX 2500M" },
@@ -364,8 +371,14 @@ static SymTabRec NVKnownChipsets[] =
   { 0x10DE0241, "GeForce 6150 LE" },
   { 0x10DE0242, "GeForce 6100" },
   { 0x10DE0244, "GeForce Go 6150" },
+  { 0x10DE0245, "Quadro NVS 210S / NVIDIA GeForce 6150LE" },
   { 0x10DE0247, "GeForce Go 6100" },
 
+  { 0x10DE03D0, "GeForce 6150SE" },
+  { 0x10DE03D1, "GeForce 6100 nForce 405" },
+  { 0x10DE03D2, "GeForce 6100 nForce 400" },
+  { 0x10DE03D5, "GeForce 6100 nForce 420" },
+
 /*************** G8x ***************/
   { 0x10DE0191, "GeForce 8800 GTX" },
   { 0x10DE0193, "GeForce 8800 GTS" },
@@ -403,160 +416,77 @@ static SymTabRec NVKnownChipsets[] =
   { 0x10DE042D, "Quadro FX 360M" },
   { 0x10DE042E, "GeForce 9300M G" },
   { 0x10DE042F, "Quadro NVS 290" },
+  { 0x10DE05E0, "GeForce GTX 295" },
   { 0x10DE05E1, "GeForce GTX 280" },
   { 0x10DE05E2, "GeForce GTX 260" },
+  { 0x10DE05E3, "GeForce GTX 285" },
+  { 0x10DE05F9, "Quadro CX" },
+  { 0x10DE05FD, "Quadro FX 5800" },
+  { 0x10DE05FE, "Quadro FX 4800" },
+  { 0x10DE05FF, "Quadro FX 3800" },
   { 0x10DE0600, "GeForce 8800 GTS 512" },
+  { 0x10DE0601, "GeForce 9800 GT" },
   { 0x10DE0602, "GeForce 8800 GT" },
   { 0x10DE0604, "GeForce 9800 GX2" },
+  { 0x10DE0605, "GeForce 9800 GT" },
   { 0x10DE0606, "GeForce 8800 GS" },
+  { 0x10DE0608, "GeForce 9800M GTX" },
   { 0x10DE0609, "GeForce 8800M GTS" },
+  { 0x10DE060B, "GeForce 9800M GT" },
   { 0x10DE060C, "GeForce 8800M GTX" },
   { 0x10DE060D, "GeForce 8800 GS" },
   { 0x10DE0610, "GeForce 9600 GSO" },
   { 0x10DE0611, "GeForce 8800 GT" },
   { 0x10DE0612, "GeForce 9800 GTX" },
-  { 0x10DE0613, "GeForce 9800 GTK+" },
+  { 0x10DE0613, "GeForce 9800 GTX+" },
   { 0x10DE0614, "GeForce 9800 GT" },
+  { 0x10DE0615, "GeForce GTS 250" },
+  { 0x10DE0617, "GeForce 9800M GTX" },
   { 0x10DE061A, "Quadro FX 3700" },
   { 0x10DE061C, "Quadro FX 3600M" },
+  { 0x10DE061E, "Quadro FX 3700M" },
   { 0x10DE0622, "GeForce 9600 GT" },
   { 0x10DE0623, "GeForce 9600 GS" },
+  { 0x10DE0625, "GeForce 9600 GSO 512" },
+  { 0x10DE0626, "GeForce GT 130" },
+  { 0x10DE0627, "GeForce GT 140" },
   { 0x10DE0628, "GeForce 9800M GTS" },
   { 0x10DE062A, "GeForce 9700M GTS" },
+  { 0x10DE062B, "GeForce 9800M GS" },
   { 0x10DE062C, "GeForce 9800M GTS" },
+  { 0x10DE0638, "Quadro FX 1800" },
+  { 0x10DE063A, "Quadro FX 2700M" },
   { 0x10DE0640, "GeForce 9500 GT" },
+  { 0x10DE0641, "GeForce 9400 GT" },
+  { 0x10DE0643, "GeForce 9500 GT" },
+  { 0x10DE0646, "GeForce GT 120" },
   { 0x10DE0647, "GeForce 9600M GT" },
   { 0x10DE0648, "GeForce 9600M GS" },
   { 0x10DE0649, "GeForce 9600M GT" },
+  { 0x10DE064A, "GeForce 9700M GT" },
   { 0x10DE064B, "GeForce 9500M G" },
+  { 0x10DE064C, "GeForce 9650M GT" },
+  { 0x10DE0656, "GeForce 9500 GT" },
+  { 0x10DE0658, "Quadro FX 380" },
+  { 0x10DE0659, "Quadro FX 580" },
+  { 0x10DE065C, "Quadro FX 770M" },
+  { 0x10DE06E0, "GeForce 9300 GE" },
   { 0x10DE06E1, "GeForce 9300 GS" },
   { 0x10DE06E4, "GeForce 8400 GS" },
   { 0x10DE06E5, "GeForce 9300M GS" },
+  { 0x10DE06E6, "GeForce G100" },
   { 0x10DE06E8, "GeForce 9200M GS" },
   { 0x10DE06E9, "GeForce 9300M GS" },
   { 0x10DE06EA, "Quadro NVS 150M" },
   { 0x10DE06EB, "Quadro NVS 160M" },
+  { 0x10DE06F8, "Quadro NVS 420" },
+  { 0x10DE06F9, "Quadro FX 370 LP" },
+  { 0x10DE06FA, "Quadro NVS 450" },
+  { 0x10DE06FD, "Quadro NVS 295" },
 
   {-1, NULL}
 };
 
-
-/*
- * List of symbols from other modules that this module references.  This
- * list is used to tell the loader that it is OK for symbols here to be
- * unresolved providing that it hasn't been told that they haven't been
- * told that they are essential via a call to xf86LoaderReqSymbols() or
- * xf86LoaderReqSymLists().  The purpose is this is to avoid warnings about
- * unresolved symbols that are not required.
- */
-
-static const char *vgahwSymbols[] = {
-    "vgaHWUnmapMem",
-    "vgaHWDPMSSet",
-    "vgaHWFreeHWRec",
-    "vgaHWGetHWRec",
-    "vgaHWGetIndex",
-    "vgaHWInit",
-    "vgaHWMapMem",
-    "vgaHWProtect",
-    "vgaHWRestore",
-    "vgaHWSave",
-    "vgaHWSaveScreen",
-    NULL
-};
-
-static const char *fbSymbols[] = {
-    "fbPictureInit",
-    "fbScreenInit",
-    NULL
-};
-
-static const char *xaaSymbols[] = {
-    "XAACopyROP",
-    "XAACreateInfoRec",
-    "XAADestroyInfoRec",
-    "XAAFallbackOps",
-    "XAAInit",
-    "XAAPatternROP",
-    NULL
-};
-
-static const char *ramdacSymbols[] = {
-    "xf86CreateCursorInfoRec",
-    "xf86DestroyCursorInfoRec",
-    "xf86InitCursor",
-    NULL
-};
-
-static const char *ddcSymbols[] = {
-    "xf86PrintEDID",
-    "xf86DoEDID_DDC2",
-    "xf86SetDDCproperties",
-    NULL
-};
-
-#ifdef XFree86LOADER
-static const char *vbeSymbols[] = {
-    "VBEInit",
-    "vbeFree",
-    "vbeDoEDID",
-    NULL
-};
-
-static const char *vbeModeSymbols[] = {
-    "VBEExtendedInit",
-    "VBEGetVBEInfo",
-    "VBEGetModePool",
-    "VBEValidateModes",
-    "VBESetModeParameters",
-    "VBEGetVBEMode",
-    "VBESetVBEMode",
-    NULL
-};
-#endif
-
-static const char *i2cSymbols[] = {
-    "xf86CreateI2CBusRec",
-    "xf86I2CBusInit",
-    NULL
-};
-
-static const char *shadowSymbols[] = {
-    "ShadowFBInit",
-    NULL
-};
-
-static const char *fbdevHWSymbols[] = {
-    "fbdevHWInit",
-    "fbdevHWUseBuildinMode",
-
-    "fbdevHWGetVidmem",
-
-    /* colormap */
-    "fbdevHWLoadPaletteWeak",
-
-    /* ScrnInfo hooks */
-    "fbdevHWAdjustFrameWeak",
-    "fbdevHWEnterVT",
-    "fbdevHWLeaveVTWeak",
-    "fbdevHWModeInit",
-    "fbdevHWSave",
-    "fbdevHWSwitchModeWeak",
-    "fbdevHWValidModeWeak",
-
-    "fbdevHWMapMMIO",
-    "fbdevHWMapVidmem",
-
-    NULL
-};
-
-static const char *int10Symbols[] = {
-    "xf86FreeInt10",
-    "xf86InitInt10",
-    NULL
-};
-
-
 #ifdef XFree86LOADER
 
 static MODULESETUPPROTO(nvSetup);
@@ -666,20 +596,6 @@ nvSetup(pointer module, pointer opts, int *errmaj, int *errmin)
 #endif
         );
 
-        /*
-         * Modules that this driver always requires may be loaded here
-         * by calling LoadSubModule().
-         */
-
-        /*
-         * Tell the loader about symbols from other modules that this module
-         * might refer to.
-         */
-        LoaderRefSymLists(vgahwSymbols, xaaSymbols, fbSymbols,
-                          ramdacSymbols, shadowSymbols,
-                          i2cSymbols, ddcSymbols, vbeSymbols,
-                          fbdevHWSymbols, int10Symbols, NULL);
-
         /*
          * The return value must be non-NULL on success even though there
          * is no TearDownProc.
@@ -808,8 +724,12 @@ NVIsG80(int chipType)
         case 0x0620:
         case 0x0630:
         case 0x0640:
+        case 0x0650:
         case 0x06e0:
         case 0x06f0:
+        case 0x0a60:
+        case 0x0a70:
+        case 0x0ca0:
             return TRUE;
     }
 
@@ -1237,11 +1157,9 @@ Bool NVI2CInit(ScrnInfoPtr pScrn)
     char *mod = "i2c";
 
     if (xf86LoadSubModule(pScrn, mod)) {
-        xf86LoaderReqSymLists(i2cSymbols,NULL);
 
         mod = "ddc";
         if(xf86LoadSubModule(pScrn, mod)) {
-            xf86LoaderReqSymLists(ddcSymbols, NULL);
             return NVDACi2cInit(pScrn);
         } 
     }
@@ -1340,15 +1258,16 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
 
     /* Initialize the card through int10 interface if needed */
     if (xf86LoadSubModule(pScrn, "int10")) {
- 	xf86LoaderReqSymLists(int10Symbols, NULL);
 #if !defined(__alpha__) && !defined(__powerpc__)
         xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Initializing int10\n");
         pNv->pInt = xf86InitInt10(pNv->pEnt->index);
 #endif
     }
    
+#ifndef XSERVER_LIBPCIACCESS
     xf86SetOperatingState(resVgaIo, pNv->pEnt->index, ResUnusedOpr);
     xf86SetOperatingState(resVgaMem, pNv->pEnt->index, ResDisableOpr);
+#endif
 
     /* Set pScrn->monitor */
     pScrn->monitor = pScrn->confScreen->monitor;
@@ -1476,8 +1395,6 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
 	return FALSE;
     }
     
-    xf86LoaderReqSymLists(vgahwSymbols, NULL);
-
     /*
      * Allocate a vgaHWRec
      */
@@ -1546,7 +1463,6 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
 	    return FALSE;
 	}
 	
-	xf86LoaderReqSymLists(fbdevHWSymbols, NULL);
 	if (!fbdevHWInit(pScrn, pNv->PciInfo, NULL)) {
 	    xf86FreeInt10(pNv->pInt);
 	    return FALSE;
@@ -1654,7 +1570,6 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
                        "enabled.\n");
             return FALSE;
         }
-        xf86LoaderReqSymLists(vbeModeSymbols, NULL);
         pNv->pVbe = VBEExtendedInit(NULL, pNv->pEnt->index,
                                     SET_BIOS_SCRATCH | RESTORE_BIOS_SCRATCH);
         if (!pNv->pVbe) return FALSE;
@@ -1729,6 +1644,7 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
     xf86DrvMsg(pScrn->scrnIndex, from, "MMIO registers at 0x%lX\n",
 	       (unsigned long)pNv->IOAddress);
      
+#ifndef XSERVER_LIBPCIACCESS
     if (xf86RegisterResources(pNv->pEnt->index, NULL, ResExclusive)) {
 	xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
 		"xf86RegisterResources() found resource conflicts\n");
@@ -1736,6 +1652,7 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
 	NVFreeRec(pScrn);
 	return FALSE;
     }
+#endif
 
     switch (pNv->Chipset & 0x0ff0) {
     case 0x0100:   /* GeForce 256 */
@@ -1986,8 +1903,6 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
 	return FALSE;
     }
 
-    xf86LoaderReqSymLists(fbSymbols, NULL);
-    
     /* Load XAA if needed */
     if (!pNv->NoAccel) {
 	if (!xf86LoadSubModule(pScrn, "xaa")) {
@@ -1995,7 +1910,6 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
 	    NVFreeRec(pScrn);
 	    return FALSE;
 	}
-	xf86LoaderReqSymLists(xaaSymbols, NULL);
     }
 
     /* Load ramdac if needed */
@@ -2005,7 +1919,6 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
 	    NVFreeRec(pScrn);
 	    return FALSE;
 	}
-	xf86LoaderReqSymLists(ramdacSymbols, NULL);
     }
 
     /* Load shadowfb if needed */
@@ -2015,7 +1928,6 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
 	    NVFreeRec(pScrn);
 	    return FALSE;
 	}
-	xf86LoaderReqSymLists(shadowSymbols, NULL);
     }
 
     pNv->CurrentLayout.bitsPerPixel = pScrn->bitsPerPixel;
